/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.button;

import javax.annotation.Nonnull;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.util.ResourceLocation;

import net.dries007.tfc.client.TFCGuiHandler;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

public class GuiButtonPlayerInventoryTab extends GuiButtonTFC
{
    private static final ResourceLocation BACKGROUND = new ResourceLocation(MOD_ID, "textures/gui/icons.png");

    private final TFCGuiHandler.Type guiType;
    private final boolean isActive;
    private int textureU, textureV, renderWidth, iconU, iconV, iconX, iconY, guiLeft;

    public GuiButtonPlayerInventoryTab(TFCGuiHandler.Type guiType, int guiLeft, int guiTop, int buttonId, boolean isActive)
    {
        super(buttonId, guiLeft + 176, guiTop, 20, 22, "");
        this.guiType = guiType;
        this.isActive = isActive;
        this.guiLeft = guiLeft;

        switch (guiType)
        {
            case INVENTORY:
                this.field_146129_i += 4;
                this.iconU = 0;
                break;
            case SKILLS:
                this.field_146129_i += 27;
                this.iconU = 32;
                break;
            case CALENDAR:
                this.field_146129_i += 50;
                this.iconU = 64;
                break;
            case NUTRITION:
                this.field_146129_i += 73;
                this.iconU = 96;
                break;
            default:
                throw new IllegalArgumentException("Invalid gui type: " + guiType);
        }

        this.renderWidth = 20;
        this.iconV = 0;
        this.textureV = 0;
        this.iconX = field_146128_h + 1;
        this.iconY = field_146129_i + 3;
        if (isActive)
        {
            this.textureU = 128;
        }
        else
        {
            this.field_146128_h -= 3;
            this.renderWidth += 3;
            this.textureU = 148;
        }
    }

    public void updateGuiLeft(int newGuiLeft)
    {
        field_146128_h -= guiLeft;
        field_146128_h += newGuiLeft;
        guiLeft = newGuiLeft;
    }

    public boolean isActive()
    {
        return isActive;
    }

    public TFCGuiHandler.Type getGuiType()
    {
        return guiType;
    }

    @Override
    public void func_191745_a(@Nonnull Minecraft mc, int mouseX, int mouseY, float partialTicks)
    {
        if (this.field_146125_m)
        {
            GlStateManager.func_179131_c(1, 1, 1, 1);
            mc.func_110434_K().func_110577_a(BACKGROUND);
            field_146123_n = mouseX >= this.field_146128_h && mouseY >= this.field_146129_i && mouseX < this.field_146128_h + this.field_146120_f && mouseY < this.field_146129_i + this.field_146121_g;
            // The button background
            func_146110_a(field_146128_h, field_146129_i, textureU, textureV, renderWidth, 22, 256, 256);
            // The icon
            func_152125_a(iconX, iconY, iconU, iconV, 32, 32, 16, 16, 256, 256);
            func_146119_b(mc, mouseX, mouseY);
        }
    }
}
