/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.gui;

import net.minecraft.client.resources.I18n;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.objects.te.TEBlastFurnace;

import static net.dries007.tfc.api.capability.heat.CapabilityItemHeat.MAX_TEMPERATURE;
import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

@SideOnly(Side.CLIENT)
public class GuiBlastFurnace extends GuiContainerTE<TEBlastFurnace>
{
    private static final ResourceLocation BLAST_FURNACE_BACKGROUND = new ResourceLocation(MOD_ID, "textures/gui/blast_furnace.png");

    public GuiBlastFurnace(Container container, InventoryPlayer playerInv, TEBlastFurnace tile)
    {
        super(container, playerInv, tile, BLAST_FURNACE_BACKGROUND);
    }

    @Override
    protected void func_146976_a(float partialTicks, int mouseX, int mouseY)
    {
        super.func_146976_a(partialTicks, mouseX, mouseY);

        int temperature = (int) (51 * tile.getField(TEBlastFurnace.FIELD_TEMPERATURE) / MAX_TEMPERATURE);
        if (temperature > 0)
        {
            func_73729_b(field_147003_i + 8, field_147009_r + 66 - temperature, 185, 32, 15, 5);
        }

        int oreCount = tile.getField(TEBlastFurnace.FIELD_ORE) * 4;
        if (oreCount > 0)
        {
            func_73729_b(field_147003_i + 40, field_147009_r + 25, 176, 0, oreCount + 1, 8);
        }

        int fuelCount = tile.getField(TEBlastFurnace.FIELD_FUEL) * 4;
        if (fuelCount > 0)
        {
            func_73729_b(field_147003_i + 40, field_147009_r + 43, 176, 0, fuelCount + 1, 8);
        }

        int meltAmount = tile.getField(TEBlastFurnace.FIELD_MELT);
        if (meltAmount > 700) meltAmount = 700;
        meltAmount = (int) (meltAmount / 8.75f);
        if (meltAmount > 0)
        {
            func_73729_b(field_147003_i + 40, field_147009_r + 61, 176, 0, meltAmount + 1, 8);
        }

        field_146289_q.func_78276_b(I18n.func_135052_a(MOD_ID + ".tooltip.blast_furnace_ore_amount"), field_147003_i + 40, field_147009_r + 17, 0x000000);
        field_146289_q.func_78276_b(I18n.func_135052_a(MOD_ID + ".tooltip.blast_furnace_fuel_amount"), field_147003_i + 40, field_147009_r + 35, 0x000000);
        field_146289_q.func_78276_b(I18n.func_135052_a(MOD_ID + ".tooltip.blast_furnace_melt_amount"), field_147003_i + 40, field_147009_r + 53, 0x000000);
    }

    @Override
    protected void func_191948_b(int mouseX, int mouseY)
    {
        if (mouseX > field_147003_i + 40 && mouseX < field_147003_i + 120 && mouseY > field_147009_r + 25 && mouseY < field_147009_r + 33)
        {
            int amount = tile.getField(TEBlastFurnace.FIELD_ORE_UNITS);
            func_146279_a(I18n.func_135052_a(MOD_ID + ".tooltip.units", amount), mouseX, mouseY);
        }
        if (mouseX > field_147003_i + 40 && mouseX < field_147003_i + 120 && mouseY > field_147009_r + 43 && mouseY < field_147009_r + 51)
        {
            int amount = tile.getField(TEBlastFurnace.FIELD_FUEL);
            func_146279_a(I18n.func_135052_a(MOD_ID + ".tooltip.units", amount), mouseX, mouseY);
        }
        if (mouseX > field_147003_i + 40 && mouseX < field_147003_i + 120 && mouseY > field_147009_r + 61 && mouseY < field_147009_r + 69)
        {
            int amount = tile.getField(TEBlastFurnace.FIELD_MELT);
            func_146279_a(I18n.func_135052_a(MOD_ID + ".tooltip.units", amount), mouseX, mouseY);
        }
        super.func_191948_b(mouseX, mouseY);
    }
}
