/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.gui;

import java.util.Map;

import net.minecraft.client.resources.I18n;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.text.TextFormatting;

import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.objects.te.TECrucible;
import net.dries007.tfc.util.Alloy;

import static net.dries007.tfc.api.capability.heat.CapabilityItemHeat.MAX_TEMPERATURE;
import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

public class GuiCrucible extends GuiContainerTE<TECrucible>
{
    private static final ResourceLocation CRUCIBLE_BACKGROUND = new ResourceLocation(MOD_ID, "textures/gui/crucible.png");

    public GuiCrucible(Container container, InventoryPlayer playerInv, TECrucible tile)
    {
        super(container, playerInv, tile, CRUCIBLE_BACKGROUND);

        this.field_147000_g = 192;
    }

    @Override
    protected void func_191948_b(int mouseX, int mouseY)
    {
        if (mouseX > field_147003_i + 128 && mouseX < field_147003_i + 137 && mouseY > field_147009_r + 5 && mouseY < field_147009_r + 107)
        {
            int amount = tile.getAlloy().getAmount();
            func_146279_a(I18n.func_135052_a(MOD_ID + ".tooltip.units", amount), mouseX, mouseY);
        }
        super.func_191948_b(mouseX, mouseY);
    }

    @Override
    protected void func_146976_a(float partialTicks, int mouseX, int mouseY)
    {
        super.func_146976_a(partialTicks, mouseX, mouseY);

        // Draw the temperature indicator
        int temperature = (int) (51 * tile.getField(TECrucible.FIELD_TEMPERATURE) / MAX_TEMPERATURE);
        if (temperature > 0)
        {
            func_73729_b(field_147003_i + 153, field_147009_r + 80 - temperature, 176, 0, 15, 5);
        }

        // Draw the filled amount
        Alloy alloy = tile.getAlloy();
        if (alloy.getAmount() > 0)
        {
            int fill = (int) (99f * alloy.getAmount() / TECrucible.CRUCIBLE_MAX_METAL_FLUID);
            func_73729_b(field_147003_i + 129, field_147009_r + 106 - fill, 191, 0, 8, fill);

            // Draw Title:
            Metal result = tile.getAlloyResult();
            String resultText = TextFormatting.UNDERLINE + I18n.func_135052_a(result.getTranslationKey()) + ":";
            field_146289_q.func_78276_b(resultText, field_147003_i + 7, field_147009_r + 7, 0x000000);

            // Draw Components
            int yPos = field_147009_r + 18;
            for (Map.Entry<Metal, Double> entry : alloy.getMetals().entrySet())
            {
                String metalName = I18n.func_135052_a(entry.getKey().getTranslationKey());
                String displayText = String.format("%s: %s%2.2f%%", metalName, TextFormatting.DARK_GREEN, 100 * entry.getValue() / alloy.getAmount());
                field_146289_q.func_78276_b(displayText, field_147003_i + 7, yPos, 0x404040);
                yPos += 9;
            }
        }
    }
}
