/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.gui;

import java.io.IOException;
import javax.annotation.Nonnull;

import net.minecraft.client.gui.GuiButton;
import net.minecraft.client.gui.GuiButtonImage;
import net.minecraft.client.gui.inventory.GuiContainer;
import net.minecraft.client.gui.inventory.GuiCrafting;
import net.minecraft.client.gui.recipebook.GuiRecipeBook;
import net.minecraft.client.gui.recipebook.IRecipeShownListener;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.resources.I18n;
import net.minecraft.inventory.ClickType;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.ContainerWorkbench;
import net.minecraft.inventory.Slot;
import net.minecraft.util.ResourceLocation;

/**
 * This is copied from {@link GuiCrafting} since it needs to use a different container implementation
 */
public class GuiInventoryCrafting extends GuiContainer implements IRecipeShownListener
{
    private static final ResourceLocation CRAFTING_TABLE_GUI_TEXTURES = new ResourceLocation("textures/gui/container/crafting_table.png");
    private final GuiRecipeBook recipeBookGui;
    private GuiButtonImage recipeButton;
    private boolean widthTooNarrow;

    public GuiInventoryCrafting(Container container)
    {
        super(container);
        this.recipeBookGui = new GuiRecipeBook();
    }

    /**
     * Adds the buttons (and other controls) to the screen in question. Called when the GUI is displayed and when the
     * window resizes, the buttonList is cleared beforehand.
     */
    public void func_73866_w_()
    {
        super.func_73866_w_();
        this.widthTooNarrow = this.field_146294_l < 379;
        this.recipeBookGui.func_194303_a(this.field_146294_l, this.field_146295_m, this.field_146297_k, this.widthTooNarrow, ((ContainerWorkbench) this.field_147002_h).field_75162_e);
        this.field_147003_i = this.recipeBookGui.func_193011_a(this.widthTooNarrow, this.field_146294_l, this.field_146999_f);
        this.recipeButton = new GuiButtonImage(10, this.field_147003_i + 5, this.field_146295_m / 2 - 49, 20, 18, 0, 168, 19, CRAFTING_TABLE_GUI_TEXTURES);
        this.field_146292_n.add(this.recipeButton);
    }

    /**
     * Draws the screen and all the components in it.
     */
    public void func_73863_a(int mouseX, int mouseY, float partialTicks)
    {
        this.func_146276_q_();

        if (this.recipeBookGui.func_191878_b() && this.widthTooNarrow)
        {
            this.func_146976_a(partialTicks, mouseX, mouseY);
            this.recipeBookGui.func_191861_a(mouseX, mouseY, partialTicks);
        }
        else
        {
            this.recipeBookGui.func_191861_a(mouseX, mouseY, partialTicks);
            super.func_73863_a(mouseX, mouseY, partialTicks);
            this.recipeBookGui.func_191864_a(this.field_147003_i, this.field_147009_r, true, partialTicks);
        }

        this.func_191948_b(mouseX, mouseY);
        this.recipeBookGui.func_191876_c(this.field_147003_i, this.field_147009_r, mouseX, mouseY);
    }

    /**
     * Draw the foreground layer for the GuiContainer (everything in front of the items)
     */
    protected void func_146979_b(int mouseX, int mouseY)
    {
        this.field_146289_q.func_78276_b(I18n.func_135052_a("container.crafting"), 28, 6, 4210752);
        this.field_146289_q.func_78276_b(I18n.func_135052_a("container.inventory"), 8, this.field_147000_g - 96 + 2, 4210752);
    }

    /**
     * Draws the background layer of this container (behind the items).
     */
    protected void func_146976_a(float partialTicks, int mouseX, int mouseY)
    {
        GlStateManager.func_179131_c(1.0F, 1.0F, 1.0F, 1.0F);
        this.field_146297_k.func_110434_K().func_110577_a(CRAFTING_TABLE_GUI_TEXTURES);
        int i = this.field_147003_i;
        int j = (this.field_146295_m - this.field_147000_g) / 2;
        this.func_73729_b(i, j, 0, 0, this.field_146999_f, this.field_147000_g);
    }

    /**
     * Called when the mouse is clicked. Args : mouseX, mouseY, clickedButton
     */
    protected void func_73864_a(int mouseX, int mouseY, int mouseButton) throws IOException
    {
        if (!this.recipeBookGui.func_191862_a(mouseX, mouseY, mouseButton))
        {
            if (!this.widthTooNarrow || !this.recipeBookGui.func_191878_b())
            {
                super.func_73864_a(mouseX, mouseY, mouseButton);
            }
        }
    }

    protected boolean func_193983_c(int p_193983_1_, int p_193983_2_, int p_193983_3_, int p_193983_4_)
    {
        boolean flag = p_193983_1_ < p_193983_3_ || p_193983_2_ < p_193983_4_ || p_193983_1_ >= p_193983_3_ + this.field_146999_f || p_193983_2_ >= p_193983_4_ + this.field_147000_g;
        return this.recipeBookGui.func_193955_c(p_193983_1_, p_193983_2_, this.field_147003_i, this.field_147009_r, this.field_146999_f, this.field_147000_g) && flag;
    }

    /**
     * Test if the 2D point is in a rectangle (relative to the GUI). Args : rectX, rectY, rectWidth, rectHeight, pointX,
     * pointY
     */
    protected boolean func_146978_c(int rectX, int rectY, int rectWidth, int rectHeight, int pointX, int pointY)
    {
        return (!this.widthTooNarrow || !this.recipeBookGui.func_191878_b()) && super.func_146978_c(rectX, rectY, rectWidth, rectHeight, pointX, pointY);
    }

    /**
     * Called when the mouse is clicked over a slot or outside the gui.
     */
    protected void func_184098_a(Slot slotIn, int slotId, int mouseButton, @Nonnull ClickType type)
    {
        super.func_184098_a(slotIn, slotId, mouseButton, type);
        this.recipeBookGui.func_191874_a(slotIn);
    }

    /**
     * Fired when a key is typed (except F11 which toggles full screen). This is the equivalent of
     * KeyListener.keyTyped(KeyEvent e). Args : character (character on the key), keyCode (lwjgl Keyboard key code)
     */
    protected void func_73869_a(char typedChar, int keyCode) throws IOException
    {
        if (!this.recipeBookGui.func_191859_a(typedChar, keyCode))
        {
            super.func_73869_a(typedChar, keyCode);
        }
    }

    /**
     * Called when the screen is unloaded. Used to disable keyboard repeat events
     */
    public void func_146281_b()
    {
        this.recipeBookGui.func_191871_c();
        super.func_146281_b();
    }

    /**
     * Called from the main game loop to update the screen.
     */
    public void func_73876_c()
    {
        super.func_73876_c();
        this.recipeBookGui.func_193957_d();
    }

    public void func_192043_J_()
    {
        this.recipeBookGui.func_193948_e();
    }

    @Nonnull
    public GuiRecipeBook func_194310_f()
    {
        return this.recipeBookGui;
    }

    /**
     * Called by the controls from the buttonList when activated. (Mouse pressed for buttons)
     */
    protected void func_146284_a(GuiButton button)
    {
        if (button.field_146127_k == 10)
        {
            this.recipeBookGui.func_193014_a(this.widthTooNarrow, ((ContainerWorkbench) this.field_147002_h).field_75162_e);
            this.recipeBookGui.func_191866_a();
            this.field_147003_i = this.recipeBookGui.func_193011_a(this.widthTooNarrow, this.field_146294_l, this.field_146999_f);
            this.recipeButton.func_191746_c(this.field_147003_i + 5, this.field_146295_m / 2 - 49);
        }
    }
}
