/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.model.animal;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.client.model.ModelQuadruped;
import net.minecraft.client.model.ModelRenderer;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.entity.Entity;
import net.minecraft.util.math.MathHelper;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC;
import net.dries007.tfc.objects.entity.animal.EntityCowTFC;

@SideOnly(Side.CLIENT)
@ParametersAreNonnullByDefault
public class ModelCowTFC extends ModelQuadruped
{
    private ModelRenderer udders;
    private ModelRenderer horn1;
    private ModelRenderer horn2;
    private ModelRenderer horn1b;
    private ModelRenderer horn2b;

    public ModelCowTFC()
    {
        super(12, 0.0F);
        this.field_78150_a = new ModelRenderer(this, 0, 0);
        this.field_78150_a.func_78790_a(-4.0F, -4.0F, -6.0F, 8, 8, 6, 0.0F);
        this.field_78150_a.func_78793_a(0.0F, 4.0F, -8.0F);

        horn1 = new ModelRenderer(this, 22, 0);
        horn1.func_78790_a(0F, 0F, 0F, 1, 3, 1, 0.15F);
        horn1.func_78793_a(-5.5F, -2.5F, -2F);
        horn1.field_78808_h = (float) -Math.PI / 2;

        horn1b = new ModelRenderer(this, 22, 0);
        horn1b.func_78790_a(0, -2.1f, -0.5f, 1, 3, 1, 0F);
        horn1b.func_78793_a(0f, 0f, 0f);
        horn1b.field_78795_f = (float) Math.PI / 3f;
        horn1b.field_78796_g = (float) -Math.PI / 12f;
        horn1.func_78792_a(horn1b);

        this.field_78150_a.func_78792_a(horn1);
        horn2 = new ModelRenderer(this, 22, 0);
        horn2.func_78790_a(0F, -3F, 0F, 1, 3, 1, 0.15F);
        horn2.func_78793_a(5.5F, -2.5F, -2F);
        horn2.field_78808_h = (float) -Math.PI / 2;

        horn2b = new ModelRenderer(this, 22, 0);
        horn2b.func_78790_a(0f, -0.8F, -0.5f, 1, 3, 1, 0F);
        horn2b.func_78793_a(0F, 0F, 0F);
        horn2b.field_78795_f = (float) -Math.PI / 3F;
        horn2b.field_78796_g = (float) -Math.PI / 12F;
        horn2.func_78792_a(horn2b);

        this.field_78150_a.func_78792_a(horn2);
        this.field_78148_b = new ModelRenderer(this, 18, 4);
        this.field_78148_b.func_78790_a(-6.0F, -10.0F, -7.0F, 12, 18, 10, 0.0F);
        this.field_78148_b.func_78793_a(0.0F, 5.0F, 2.0F);
        this.udders = new ModelRenderer(this, 18, 4);
        this.udders.func_78793_a(0.0F, 5.0F, 2.0F);
        this.udders.func_78784_a(52, 0).func_78789_a(-2.0F, 0.0F, -8.0F, 4, 6, 1);
    }

    @Override
    public void func_78088_a(@Nonnull Entity entity, float par2, float par3, float par4, float par5, float par6, float par7)
    {
        this.func_78087_a(par2, par3, par4, par5, par6, par7, entity);

        EntityCowTFC cow = ((EntityCowTFC) entity);

        float percent = cow.getPercentToAdulthood();

        float ageScale = 2.0F - percent;
        float ageHeadScale = (float) Math.pow(1 / ageScale, 0.66);
        GlStateManager.func_179094_E();

        GlStateManager.func_179109_b(0.0F, 0.75f - (0.75f * percent), 0f);
        GlStateManager.func_179152_a(ageHeadScale, ageHeadScale, ageHeadScale);
        GlStateManager.func_179109_b(0.0F, 0, 0.1875f - (0.1875f * percent));
        if (percent < 0.5)
        {
            horn1.field_78807_k = true;
            horn2.field_78807_k = true;
            if (percent < 0.75)
            {
                horn1b.field_78807_k = true;
                horn2b.field_78807_k = true;
            }
        }

        if (cow.getGender() == EntityAnimalTFC.Gender.MALE)
        {
            udders.field_78807_k = true;
        }
        else
        {
            horn1b.field_78807_k = true;
            horn2b.field_78807_k = true;
        }
        field_78150_a.func_78785_a(par7);
        GlStateManager.func_179121_F();
        GlStateManager.func_179094_E();
        GlStateManager.func_179109_b(0.0F, 0.75f - (0.75f * percent), 0f);
        GlStateManager.func_179152_a(1 / ageScale, 1 / ageScale, 1 / ageScale);

        field_78148_b.func_78785_a(par7);
        udders.func_78785_a(par7);
        field_78149_c.func_78785_a(par7);
        field_78146_d.func_78785_a(par7);
        field_78147_e.func_78785_a(par7);
        field_78144_f.func_78785_a(par7);
        GlStateManager.func_179121_F();
    }

    @Override
    public void func_78087_a(float par1, float par2, float par3, float par4, float par5, float par6, Entity ent)
    {
        this.field_78150_a.field_78795_f = par5 / (180F / (float) Math.PI);
        this.field_78150_a.field_78796_g = par4 / (180F / (float) Math.PI);
        this.field_78148_b.field_78795_f = (float) Math.PI / 2F;
        this.udders.field_78795_f = (float) Math.PI / 2F;
        this.field_78149_c.field_78795_f = MathHelper.func_76134_b(par1 * 0.6662F) * 1.4F * par2;
        this.field_78146_d.field_78795_f = MathHelper.func_76134_b(par1 * 0.6662F + (float) Math.PI) * 1.4F * par2;
        this.field_78147_e.field_78795_f = MathHelper.func_76134_b(par1 * 0.6662F + (float) Math.PI) * 1.4F * par2;
        this.field_78144_f.field_78795_f = MathHelper.func_76134_b(par1 * 0.6662F) * 1.4F * par2;
        horn1.field_78795_f = 0F;
        horn2.field_78795_f = 0F;
        horn1.field_78807_k = false;
        horn1b.field_78807_k = false;
        horn2.field_78807_k = false;
        horn2b.field_78807_k = false;
        udders.field_78807_k = false;
    }
}
