/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.command;

import java.util.Collections;
import java.util.List;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.command.CommandBase;
import net.minecraft.command.CommandException;
import net.minecraft.command.ICommandSender;
import net.minecraft.command.WrongUsageException;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.TextComponentString;

import net.dries007.tfc.world.classic.worldgen.WorldGenOreVeins;
import net.dries007.tfc.world.classic.worldgen.vein.Vein;
import net.dries007.tfc.world.classic.worldgen.vein.VeinRegistry;
import net.dries007.tfc.world.classic.worldgen.vein.VeinType;

@ParametersAreNonnullByDefault
public class CommandFindVeins extends CommandBase
{
    @Override
    @Nonnull
    public String func_71517_b()
    {
        return "findveins";
    }

    @Override
    @Nonnull
    public String func_71518_a(ICommandSender sender)
    {
        return "/findveins [all|<vein name>] <radius> -> Finds all instances of a specific vein, or all veins within a certian chunk radius";
    }

    @Override
    public void func_184881_a(MinecraftServer server, ICommandSender sender, String[] args) throws CommandException
    {
        if (args.length != 2) throw new WrongUsageException("2 arguments required.");
        if (sender.func_174793_f() == null) throw new WrongUsageException("Can only be used by a player");

        sender.func_145747_a(new TextComponentString("Veins Found: "));

        final int radius = func_175764_a(args[1], 1, 1000);
        final List<Vein> veins = WorldGenOreVeins.getNearbyVeins(sender.func_174793_f().field_70176_ah, sender.func_174793_f().field_70164_aj, sender.func_130014_f_().func_72905_C(), radius);
        if (!args[0].equals("all"))
        {
            final VeinType type = VeinRegistry.INSTANCE.getVein(args[0]);
            if (type == null)
            {
                throw new WrongUsageException("Vein supplied does not match 'all' or any valid vein names. Use /veininfo to see valid vein names");
            }
            // Search for veins matching type
            veins.removeIf(x -> x.type != type);
        }
        veins.forEach(x -> sender.func_145747_a(new TextComponentString("> Vein: " + x.type + " at " + x.pos)));
    }

    @Override
    public int func_82362_a()
    {
        return 2;
    }

    @Override
    @Nonnull
    public List<String> func_184883_a(MinecraftServer server, ICommandSender sender, String[] args, @Nullable BlockPos targetPos)
    {
        if (args.length == 1)
        {
            return func_175762_a(args, VeinRegistry.INSTANCE.keySet());
        }
        return Collections.emptyList();
    }
}

