/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyEnum;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.objects.items.ItemAnimalHide;
import net.dries007.tfc.objects.te.TEPlacedHide;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;

/**
 * Due to implementation, this will only ever be a soaked hide -> scraped hide
 * Placement is restricted to the TFC item.
 */
@ParametersAreNonnullByDefault
public class BlockPlacedHide extends Block
{
    public static final PropertyEnum<ItemAnimalHide.HideSize> SIZE = PropertyEnum.func_177709_a("size", ItemAnimalHide.HideSize.class);

    BlockPlacedHide()
    {
        super(Material.field_151594_q);
        func_149711_c(0.2f);

        func_180632_j(func_176194_O().func_177621_b().func_177226_a(SIZE, ItemAnimalHide.HideSize.MEDIUM));
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_185481_k(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149730_j(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public IBlockState func_176203_a(int meta)
    {
        return func_176223_P().func_177226_a(SIZE, ItemAnimalHide.HideSize.valueOf(meta));
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(SIZE).ordinal();
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149637_q(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149721_r(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return BlockPlacedItem.PLACED_ITEM_AABB;
    }

    @SuppressWarnings("deprecation")
    @Nullable
    @Override
    public AxisAlignedBB func_180646_a(IBlockState blockState, IBlockAccess worldIn, BlockPos pos)
    {
        return BlockPlacedItem.PLACED_ITEM_AABB;
    }

    @SuppressWarnings("deprecation")
    @SideOnly(Side.CLIENT)
    @Override
    @Nonnull
    public AxisAlignedBB func_180640_a(IBlockState state, World worldIn, BlockPos pos)
    {
        return BlockPlacedItem.PLACED_ITEM_AABB;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    public void func_180663_b(World worldIn, BlockPos pos, IBlockState state)
    {
        InventoryHelper.func_180173_a(worldIn, pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(), getItemStackDropped(worldIn, pos, state));
        super.func_180663_b(worldIn, pos, state);
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        ItemStack stack = playerIn.func_184586_b(hand);
        if (OreDictionaryHelper.doesStackMatchOre(stack, "knife"))
        {
            if (!worldIn.field_72995_K)
            {
                stack.func_77972_a(1, playerIn);
            }
            TEPlacedHide tile = Helpers.getTE(worldIn, pos, TEPlacedHide.class);
            if (tile != null)
            {
                tile.onClicked(hitX, hitZ);
            }
            return true;
        }
        return super.func_180639_a(worldIn, pos, state, playerIn, hand, facing, hitX, hitY, hitZ);
    }

    @Nonnull
    @Override
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, SIZE);
    }

    @Override
    public boolean isNormalCube(IBlockState state, IBlockAccess world, BlockPos pos)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean isSideSolid(IBlockState base_state, IBlockAccess world, BlockPos pos, EnumFacing side)
    {
        return false;
    }

    @Override
    public boolean hasTileEntity(IBlockState state)
    {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TEPlacedHide();
    }

    @Override
    public void getDrops(NonNullList<ItemStack> drops, IBlockAccess world, BlockPos pos, IBlockState state, int fortune)
    {
        // No drops here because they are done in breakBlock due to needing the TE
    }

    @Override
    @Nonnull
    public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player)
    {
        return getItemStackDropped(world, pos, state);
    }

    private ItemStack getItemStackDropped(IBlockAccess world, BlockPos pos, IBlockState state)
    {
        TEPlacedHide tile = Helpers.getTE(world, pos, TEPlacedHide.class);
        if (tile != null && tile.isComplete())
        {
            return new ItemStack(ItemAnimalHide.get(ItemAnimalHide.HideType.SCRAPED, state.func_177229_b(SIZE)));
        }
        return new ItemStack(ItemAnimalHide.get(ItemAnimalHide.HideType.SOAKED, state.func_177229_b(SIZE)));
    }
}
