/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.agriculture;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.BlockBush;
import net.minecraft.block.IGrowable;
import net.minecraft.block.SoundType;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.api.types.IFruitTree;
import net.dries007.tfc.objects.te.TETickCounter;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;
import net.dries007.tfc.util.calendar.ICalendar;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockFruitTreeSapling extends BlockBush implements IGrowable
{
    private static final AxisAlignedBB SAPLING_AABB = new AxisAlignedBB(0.1, 0, 0.1, 0.9, 0.9, 0.9);

    private static final Map<IFruitTree, BlockFruitTreeSapling> MAP = new HashMap<>();

    public static BlockFruitTreeSapling get(IFruitTree tree)
    {
        return MAP.get(tree);
    }

    public final IFruitTree tree;

    public BlockFruitTreeSapling(IFruitTree tree)
    {
        if (MAP.put(tree, this) != null) throw new IllegalStateException("There can only be one.");
        this.tree = tree;
        func_149672_a(SoundType.field_185850_c);
        func_149711_c(0.0F);
        func_149675_a(true);
        OreDictionaryHelper.register(this, "tree", "sapling");
        OreDictionaryHelper.register(this, "tree", "sapling", tree.getName());
        Blocks.field_150480_ab.func_180686_a(this, 5, 20);
    }

    @Override
    public void func_180645_a(World world, BlockPos pos, IBlockState state, Random random)
    {
        super.func_180650_b(world, pos, state, random);

        if (!world.field_72995_K)
        {
            TETickCounter te = Helpers.getTE(world, pos, TETickCounter.class);
            if (te != null)
            {
                float temp = ClimateTFC.getTemp(world, pos);
                float rainfall = ChunkDataTFC.getRainfall(world, pos);
                long hours = te.getTicksSinceUpdate() / ICalendar.TICKS_IN_HOUR;
                if (hours > tree.getGrowthTime() && tree.isValidForGrowth(temp, rainfall))
                {
                    te.resetCounter();
                    func_176474_b(world, random, pos, state);
                }
            }
        }
    }

    @Override
    public void func_176213_c(World worldIn, BlockPos pos, IBlockState state)
    {
        TETickCounter te = Helpers.getTE(worldIn, pos, TETickCounter.class);
        if (te != null)
        {
            te.resetCounter();
        }
    }

    @Override
    public boolean hasTileEntity(IBlockState state)
    {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TETickCounter();
    }


    @SuppressWarnings("deprecation")
    @Override
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return SAPLING_AABB;
    }

    @Override
    public boolean func_176473_a(World world, BlockPos blockPos, IBlockState iBlockState, boolean b)
    {
        return true;
    }

    @Override
    public boolean func_180670_a(World world, Random random, BlockPos blockPos, IBlockState iBlockState)
    {
        return true; //Only on sapling tho, so trunk still has to grow
    }

    @Override
    public void func_176474_b(World world, Random random, BlockPos blockPos, IBlockState blockState)
    {
        if (!world.field_72995_K)
        {
            world.func_175656_a(blockPos, BlockFruitTreeTrunk.get(this.tree).func_176223_P());
            if (world.func_180495_p(blockPos.func_177984_a()).func_185904_a().func_76222_j())
            {
                world.func_175656_a(blockPos.func_177984_a(), BlockFruitTreeLeaves.get(tree).func_176223_P().func_177226_a(BlockFruitTreeLeaves.HARVESTABLE, false));
            }
        }
    }
}
