/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.metal;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyDirection;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.client.TFCGuiHandler;
import net.dries007.tfc.client.TFCSounds;
import net.dries007.tfc.objects.items.metal.ItemAnvil;
import net.dries007.tfc.objects.te.TEAnvilTFC;
import net.dries007.tfc.objects.te.TEInventory;
import net.dries007.tfc.util.Helpers;

import static net.dries007.tfc.objects.te.TEAnvilTFC.SLOT_HAMMER;

@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public class BlockAnvilTFC extends Block
{
    public static final PropertyDirection AXIS = PropertyDirection.func_177712_a("axis", EnumFacing.Plane.HORIZONTAL);
    private static final Map<Metal, BlockAnvilTFC> MAP = new HashMap<>();
    private static final AxisAlignedBB AABB_Z = new AxisAlignedBB(0.1875, 0, 0, 0.8125, 0.6875, 1);
    private static final AxisAlignedBB AABB_X = new AxisAlignedBB(0, 0, 0.1875, 1, 0.6875, 0.8125);

    public static BlockAnvilTFC get(Metal metal)
    {
        return MAP.get(metal);
    }

    public static ItemStack get(Metal metal, int amount)
    {
        return new ItemStack(MAP.get(metal), amount);
    }

    private final Metal metal;

    public BlockAnvilTFC(Metal metal)
    {
        super(Material.field_151573_f);

        this.metal = metal;
        if (MAP.put(metal, this) != null) throw new IllegalStateException("There can only be one.");

        func_149711_c(4.0F);
        func_149752_b(10F);
        setHarvestLevel("pickaxe", 0);

        func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(AXIS, EnumFacing.NORTH));
    }

    @Override
    @SuppressWarnings("deprecation")
    public IBlockState func_176203_a(int meta)
    {
        return this.func_176223_P().func_177226_a(AXIS, EnumFacing.func_176731_b(meta));
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(AXIS).func_176736_b();
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return state.func_177229_b(AXIS).func_176740_k() == EnumFacing.Axis.X ? AABB_Z : AABB_X;
    }

    @SideOnly(Side.CLIENT)
    @Override
    @SuppressWarnings("deprecation")
    public AxisAlignedBB func_180640_a(IBlockState state, World worldIn, BlockPos pos)
    {
        return func_185496_a(state, worldIn, pos).func_186670_a(pos);
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    public Item func_180660_a(IBlockState state, Random rand, int fortune)
    {
        return ItemAnvil.get(metal, Metal.ItemType.ANVIL);
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        TEAnvilTFC te = Helpers.getTE(worldIn, pos, TEAnvilTFC.class);
        if (te == null)
        {
            return false;
        }
        IItemHandler cap = te.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null);
        if (cap == null)
        {
            return false;
        }
        ItemStack heldItem = playerIn.func_184586_b(hand);
        if (playerIn.func_70093_af())
        {
            // Extract requires an empty hand
            if (heldItem.func_190926_b())
            {
                // Only check the input slots
                for (int i = 0; i < 2; i++)
                {
                    ItemStack stack = cap.getStackInSlot(i);
                    if (!stack.func_190926_b())
                    {
                        // Give the item to player in the main hand
                        ItemStack result = cap.extractItem(i, 1, false);
                        playerIn.func_184611_a(hand, result);
                        return true;
                    }
                }
            }
            // Welding requires a hammer
            else if (te.isItemValid(SLOT_HAMMER, heldItem))
            {
                if (te.attemptWelding(playerIn))
                {
                    // Valid welding occurred.
                    worldIn.func_184133_a(null, pos, TFCSounds.ANVIL_IMPACT, SoundCategory.PLAYERS, 1.0f, 1.0f);
                    return true;
                }
            }
        }
        else
        {
            // Not sneaking = insert items
            ItemStack stack = playerIn.func_184586_b(hand);
            if (!stack.func_190926_b())
            {
                for (int i = 0; i <= 4; i++)
                {
                    // Check the input slots and flux. Do NOT check the hammer slot
                    if (i == SLOT_HAMMER) continue;
                    // Try to insert an item
                    // Do not insert hammers into the input slots
                    if (te.isItemValid(i, stack) && cap.getStackInSlot(i).func_190926_b() && !te.isItemValid(SLOT_HAMMER, stack))
                    {
                        ItemStack result = cap.insertItem(i, stack, false);
                        playerIn.func_184611_a(hand, result);
                        TerraFirmaCraft.getLog().info("Inserted {} into slot {}", stack.func_82833_r(), i);
                        return true;
                    }
                }
            }

            // No insertion happened, so try and open GUI
            if (!worldIn.field_72995_K)
            {
                TFCGuiHandler.openGui(worldIn, pos, playerIn, TFCGuiHandler.Type.ANVIL);
            }
            return true;
        }
        return false;
    }

    @Override
    public void func_180657_a(World worldIn, EntityPlayer player, BlockPos pos, IBlockState state, @Nullable TileEntity te, ItemStack stack)
    {
        if (!worldIn.field_72995_K && te instanceof TEInventory)
        {
            ((TEInventory) te).onBreakBlock(worldIn, pos);
        }
        super.func_180657_a(worldIn, player, pos, state, te, stack);
    }

    @Override
    public BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, AXIS);
    }

    @Override
    public boolean isNormalCube(IBlockState state, IBlockAccess world, BlockPos pos)
    {
        return false;
    }

    @Override
    public boolean hasTileEntity(IBlockState state)
    {
        return true;
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(World world, IBlockState state)
    {
        return new TEAnvilTFC();
    }

    @Override
    public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player)
    {
        return new ItemStack(ItemAnvil.get(metal, Metal.ItemType.ANVIL));
    }

    public Metal getMetal()
    {
        return metal;
    }
}
