/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.plants;

import java.util.HashMap;
import java.util.Map;
import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.properties.PropertyDirection;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.Mirror;
import net.minecraft.util.Rotation;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import net.dries007.tfc.api.types.Plant;
import net.dries007.tfc.objects.blocks.wood.BlockLogTFC;
import net.dries007.tfc.world.classic.ClimateTFC;
import net.dries007.tfc.world.classic.chunkdata.ChunkDataTFC;

import net.minecraft.block.Block.EnumOffsetType;

@ParametersAreNonnullByDefault
public class BlockEpiphyteTFC extends BlockPlantTFC
{
    private static final PropertyDirection FACING = PropertyDirection.func_177714_a("facing");
    private static final AxisAlignedBB PLANT_UP_AABB = new AxisAlignedBB(0.25D, 0.0D, 0.25D, 0.75D, 0.75D, 0.75D);
    private static final AxisAlignedBB PLANT_DOWN_AABB = new AxisAlignedBB(0.25D, 0.25D, 0.25D, 0.75D, 1.0D, 0.75D);
    private static final AxisAlignedBB PLANT_NORTH_AABB = new AxisAlignedBB(0.0D, 0.0D, 0.25D, 1.0D, 1.0D, 1.0D);
    private static final AxisAlignedBB PLANT_SOUTH_AABB = new AxisAlignedBB(0.0D, 0.0D, 0.0D, 1.0D, 1.0D, 0.75D);
    private static final AxisAlignedBB PLANT_WEST_AABB = new AxisAlignedBB(0.25D, 0.0D, 0.0D, 1.0D, 1.0D, 1.0D);
    private static final AxisAlignedBB PLANT_EAST_AABB = new AxisAlignedBB(0.0D, 0.0D, 0.0D, 0.75D, 1.0D, 1.0D);

    private static final Map<Plant, BlockEpiphyteTFC> MAP = new HashMap<>();

    public static BlockEpiphyteTFC get(Plant plant)
    {
        return MAP.get(plant);
    }

    public BlockEpiphyteTFC(Plant plant)
    {
        super(plant);
        if (MAP.put(plant, this) != null) throw new IllegalStateException("There can only be one.");
    }

    @Override
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        this.onNeighborChangeInternal(worldIn, pos, state);
    }

    @Nonnull
    @Override
    public IBlockState func_176203_a(int meta)
    {
        return this.func_176223_P().func_177226_a(FACING, EnumFacing.func_82600_a(meta));
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(FACING).func_176745_a();
    }

    @Override
    public void func_176213_c(World world, BlockPos pos, IBlockState state)
    {
        world.func_175656_a(pos, state.func_177226_a(DAYPERIOD, getDayPeriod()).func_177226_a(growthStageProperty, plant.getStageForMonth()));
        func_176475_e(world, pos, state);
    }

    @Override
    @Nonnull
    public Block.EnumOffsetType func_176218_Q()
    {
        return EnumOffsetType.NONE;
    }

    @Override
    public boolean func_176196_c(World worldIn, BlockPos pos)
    {
        for (EnumFacing enumfacing : FACING.func_177700_c())
        {
            if (this.canPlaceAt(worldIn, pos, enumfacing))
            {
                return worldIn.func_180495_p(pos).func_177230_c() != this;
            }
        }

        return false;
    }

    @Override
    protected boolean func_185514_i(IBlockState state)
    {
        return true;
    }

    @Override
    public boolean func_180671_f(World worldIn, BlockPos pos, IBlockState state)
    {
        for (EnumFacing enumfacing : FACING.func_177700_c())
        {
            if (this.canPlaceAt(worldIn, pos, enumfacing))
            {
                return plant.isValidTemp(ClimateTFC.getHeightAdjustedTemp(worldIn, pos)) && plant.isValidRain(ChunkDataTFC.getRainfall(worldIn, pos));
            }
        }

        return false;
    }

    @Override
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        switch (state.func_177229_b(FACING))
        {
            case EAST:
                return PLANT_EAST_AABB;
            case WEST:
                return PLANT_WEST_AABB;
            case SOUTH:
                return PLANT_SOUTH_AABB;
            case NORTH:
                return PLANT_NORTH_AABB;
            case DOWN:
                return PLANT_DOWN_AABB;
            default:
                return PLANT_UP_AABB;
        }
    }

    @Nonnull
    protected BlockStateContainer createPlantBlockState()
    {
        return new BlockStateContainer(this, FACING, growthStageProperty, DAYPERIOD, AGE);
    }

    @SuppressWarnings("deprecation")
    @Nonnull
    @Override
    public IBlockState func_185499_a(IBlockState state, Rotation rot)
    {
        return state.func_177226_a(FACING, rot.func_185831_a(state.func_177229_b(FACING)));
    }

    @SuppressWarnings("deprecation")
    @Nonnull
    @Override
    public IBlockState func_185471_a(IBlockState state, Mirror mirrorIn)
    {
        return state.func_185907_a(mirrorIn.func_185800_a(state.func_177229_b(FACING)));
    }

    @SuppressWarnings("deprecation")
    @Nonnull
    @Override
    public IBlockState func_180642_a(World worldIn, BlockPos pos, EnumFacing facing, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer)
    {
        if (this.canPlaceAt(worldIn, pos, facing))
        {
            return this.func_176223_P().func_177226_a(FACING, facing);
        }
        else
        {
            for (EnumFacing enumfacing : EnumFacing.Plane.HORIZONTAL)
            {
                if (this.canPlaceAt(worldIn, pos, enumfacing))
                {
                    return this.func_176223_P().func_177226_a(FACING, enumfacing);
                }
            }

            return this.func_176223_P();
        }
    }

    public IBlockState getStateForWorldGen(World worldIn, BlockPos pos)
    {
        for (EnumFacing enumfacing : EnumFacing.Plane.HORIZONTAL)
        {
            if (this.canPlaceAt(worldIn, pos, enumfacing))
            {
                return this.func_176223_P().func_177226_a(FACING, enumfacing);
            }
        }
        for (EnumFacing enumfacing : EnumFacing.Plane.VERTICAL)
        {
            if (this.canPlaceAt(worldIn, pos, enumfacing))
            {
                return this.func_176223_P().func_177226_a(FACING, enumfacing);
            }
        }

        return this.func_176223_P();
    }

    private void onNeighborChangeInternal(World worldIn, BlockPos pos, IBlockState state)
    {
        if (this.checkForDrop(worldIn, pos, state))
        {
            EnumFacing facing = state.func_177229_b(FACING);
            EnumFacing.Axis axis = facing.func_176740_k();
            BlockPos blockpos = pos.func_177972_a(facing.func_176734_d());
            boolean flag = false;

            if (axis.func_176722_c() && worldIn.func_180495_p(blockpos).func_193401_d(worldIn, blockpos, facing) != BlockFaceShape.SOLID)
            {
                flag = true;
            }
            else if (axis.func_176720_b() && !this.canPlaceOn(worldIn, blockpos))
            {
                flag = true;
            }

            if (flag)
            {
                worldIn.func_175655_b(pos, true);
            }
        }
    }

    private boolean checkForDrop(World worldIn, BlockPos pos, IBlockState state)
    {
        if (state.func_177230_c() == this && this.canPlaceAt(worldIn, pos, state.func_177229_b(FACING)))
        {
            return true;
        }
        else
        {
            if (worldIn.func_180495_p(pos).func_177230_c() == this)
            {
                func_176475_e(worldIn, pos, state);
            }

            return false;
        }
    }

    private boolean canPlaceOn(World worldIn, BlockPos pos)
    {
        IBlockState state = worldIn.func_180495_p(pos);
        return state.func_177230_c() instanceof BlockLogTFC;
    }

    private boolean canPlaceAt(World worldIn, BlockPos pos, EnumFacing facing)
    {
        BlockPos blockpos = pos.func_177972_a(facing.func_176734_d());
        IBlockState iblockstate = worldIn.func_180495_p(blockpos);
        BlockFaceShape blockfaceshape = iblockstate.func_193401_d(worldIn, blockpos, facing);

        return this.canPlaceOn(worldIn, blockpos) && blockfaceshape == BlockFaceShape.SOLID;
    }
}
