/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.stone;

import java.util.Arrays;
import java.util.Random;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.api.types.Rock;
import net.dries007.tfc.objects.entity.EntityFallingBlockTFC;
import net.dries007.tfc.util.IFallingBlock;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockRockVariantFallable extends BlockRockVariant implements IFallingBlock
{
    public BlockRockVariantFallable(Rock.Type type, Rock rock)
    {
        super(type, rock);
    }

    @SideOnly(Side.CLIENT)
    @Override
    public void func_180655_c(IBlockState stateIn, World worldIn, BlockPos pos, Random rand)
    {
        if (!this.type.canFall()) return;
        if (rand.nextInt(16) != 0) return;
        if (shouldFall(worldIn, pos, pos))
        {
            double d0 = (double) ((float) pos.func_177958_n() + rand.nextFloat());
            double d1 = (double) pos.func_177956_o() - 0.05D;
            double d2 = (double) ((float) pos.func_177952_p() + rand.nextFloat());
            worldIn.func_175688_a(EnumParticleTypes.FALLING_DUST, d0, d1, d2, 0.0D, 0.0D, 0.0D, Block.func_176210_f(stateIn));
        }
    }

    @SuppressWarnings("deprecation")
    @Override
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        super.func_189540_a(state, worldIn, pos, blockIn, fromPos);
        if (checkFalling(worldIn, pos, state))
        {
            onRockSlide(worldIn, pos);
        }
    }

    @Override
    public void func_176213_c(World worldIn, BlockPos pos, IBlockState state)
    {
        super.func_176213_c(worldIn, pos, state);
        if (checkFalling(worldIn, pos, state))
        {
            onRockSlide(worldIn, pos);
        }
    }

    // What is the position that the block will fall at?
    @Nullable
    @Override
    public BlockPos getFallablePos(World world, BlockPos pos)
    {
        if (type.canFall() && shouldFall(world, pos, pos))
        {
            return checkAreaClear(world, pos);
        }
        if (type.canFallHorizontal())
        {
            // Check if supported by at least two horizontals, or one on top
            if (world.func_180495_p(pos.func_177984_a()).func_185915_l())
            {
                return null;
            }

            EnumFacing[] faces = Arrays.stream(EnumFacing.field_176754_o)
                .filter(x -> world.func_180495_p(pos.func_177972_a(x)).func_185915_l())
                .toArray(EnumFacing[]::new);

            if (faces.length >= 2)
            {
                return null;
            }

            // Check if it can fall
            faces = Arrays.stream(EnumFacing.field_176754_o)
                .filter(x -> shouldFall(world, pos.func_177972_a(x), pos) && canFallThrough(world.func_180495_p(pos.func_177972_a(x))))
                .toArray(EnumFacing[]::new);

            if (faces.length >= 1)
            {
                return checkAreaClear(world, pos.func_177972_a(faces[RANDOM.nextInt(faces.length)]));
            }
        }
        return null;
    }

    @Nullable
    private BlockPos checkAreaClear(World world, BlockPos pos)
    {
        // Check that there are no entities in the area, otherwise it would collide with them
        if (!world.func_72872_a(EntityFallingBlockTFC.class, new AxisAlignedBB(pos, pos.func_177982_a(1, 1, 1))).isEmpty())
        {
            // If we can't fall due to a collision, wait for the block to move out of the way and try again later
            world.func_175684_a(pos, this, 20);
            return null;
        }
        return pos;
    }
}
