/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.wood;


import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.BlockContainer;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.MapColor;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumBlockRenderType;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import net.dries007.tfc.api.capability.size.IItemSize;
import net.dries007.tfc.api.capability.size.Size;
import net.dries007.tfc.api.capability.size.Weight;
import net.dries007.tfc.api.types.Tree;
import net.dries007.tfc.objects.te.TELoom;
import net.dries007.tfc.util.Helpers;

import static net.minecraft.block.BlockHorizontal.FACING;
import static net.minecraft.block.material.Material.WOOD;

@ParametersAreNonnullByDefault
public class BlockLoom extends BlockContainer implements IItemSize
{
    protected static final AxisAlignedBB LOOM_EAST_AABB = new AxisAlignedBB(0.125D, 0.0D, 0.0625D, 0.5625D, 1.0D, 0.9375D);
    protected static final AxisAlignedBB LOOM_WEST_AABB = new AxisAlignedBB(0.4375D, 0.0D, 0.0625D, 0.875D, 1.0D, 0.9375D);
    protected static final AxisAlignedBB LOOM_SOUTH_AABB = new AxisAlignedBB(0.0625D, 0.0D, 0.125D, 0.9375D, 1.0D, 0.5625D);
    protected static final AxisAlignedBB LOOM_NORTH_AABB = new AxisAlignedBB(0.0625D, 0.0D, 0.4375D, 0.9375D, 1.0D, 0.875D);

    public Tree wood;

    public BlockLoom(Tree wood)
    {
        super(field_151575_d, MapColor.field_151660_b);
        this.wood = wood;
        func_149672_a(SoundType.field_185848_a);
        setHarvestLevel("axe", 0);
        func_149711_c(0.5f);
        func_149752_b(3f);
        this.func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(field_185512_D, EnumFacing.NORTH));
    }

    @Nonnull
    @Override
    public Size getSize(@Nonnull ItemStack stack)
    {
        return Size.LARGE;
    }

    @Nonnull
    @Override
    public Weight getWeight(@Nonnull ItemStack stack)
    {
        return Weight.HEAVY;
    }

    @Nullable
    @Override
    public TileEntity func_149915_a(@Nonnull World worldIn, int meta)
    {
        return new TELoom();
    }

    @Override
    @SuppressWarnings("deprecation")
    @Nonnull
    public IBlockState func_176203_a(int meta)
    {
        return this.func_176223_P().func_177226_a(field_185512_D, EnumFacing.func_176731_b(meta));
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(field_185512_D).func_176736_b();
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @Override
    @SuppressWarnings("deprecation")
    @Nonnull
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        switch (state.func_177229_b(field_185512_D))
        {
            case NORTH:
            default:
                return LOOM_NORTH_AABB;
            case SOUTH:
                return LOOM_SOUTH_AABB;
            case WEST:
                return LOOM_WEST_AABB;
            case EAST:
                return LOOM_EAST_AABB;
        }
    }

    @Override
    @SuppressWarnings("deprecation")
    @Nonnull
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return BlockFaceShape.UNDEFINED;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        TELoom te = Helpers.getTE(worldIn, pos, TELoom.class);
        if (te != null)
        {
            return te.onRightClick(playerIn);
        }
        return true;
    }

    @Override
    @SuppressWarnings("deprecation")
    @Nonnull
    public IBlockState func_180642_a(World worldIn, BlockPos pos, EnumFacing facing, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer)
    {
        if (facing.func_176740_k() == EnumFacing.Axis.Y)
        {
            facing = placer.func_174811_aO().func_176734_d();
        }
        return func_176223_P().func_177226_a(field_185512_D, facing);
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, field_185512_D);
    }

    @Override
    @SuppressWarnings("deprecation")
    @Nonnull
    public EnumBlockRenderType func_149645_b(IBlockState state)
    {
        return EnumBlockRenderType.MODEL;
    }

    @Override
    public void func_180663_b(World worldIn, @Nonnull BlockPos pos, @Nonnull IBlockState state)
    {
        TELoom te = Helpers.getTE(worldIn, pos, TELoom.class);
        if (te != null)
        {
            te.onBreakBlock(worldIn, pos);
        }
        super.func_180663_b(worldIn, pos, state);
    }
}
