/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.container;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ClickType;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandler;
import net.minecraftforge.items.ItemStackHandler;

@ParametersAreNonnullByDefault
public abstract class ContainerItemStack extends Container
{
    protected final ItemStack stack;
    protected final EntityPlayer player;
    protected int itemIndex;
    protected boolean isOffhand;

    ContainerItemStack(InventoryPlayer playerInv, ItemStack stack)
    {
        super();
        this.player = playerInv.field_70458_d;
        this.stack = stack;

        if (stack == player.func_184614_ca())
        {
            this.itemIndex = playerInv.field_70461_c + 27; // Mainhand opened inventory
            this.isOffhand = false;
        }
        else
        {
            this.itemIndex = -100; // Offhand, so ignore this rule
            this.isOffhand = true;
        }

        addContainerSlots();
        addPlayerInventorySlots(playerInv);
    }

    @Override
    @Nonnull
    public ItemStack func_82846_b(EntityPlayer player, int index)
    {
        // Slot that was clicked
        Slot slot = field_75151_b.get(index);

        ItemStack itemstack;

        if (slot == null || !slot.func_75216_d())
            return ItemStack.field_190927_a;

        if (index == itemIndex)
            return ItemStack.field_190927_a;

        ItemStack itemstack1 = slot.func_75211_c();
        itemstack = itemstack1.func_77946_l();

        // Begin custom transfer code here

        int containerSlots = field_75151_b.size() - player.field_71071_by.field_70462_a.size(); // number of slots in the container

        if (index < containerSlots)
        {
            // Transfer out of the container
            if (!this.func_75135_a(itemstack1, containerSlots, field_75151_b.size(), true))
            {
                // Don't transfer anything
                return ItemStack.field_190927_a;
            }
            //tile.setAndUpdateSlots(index);
        }
        // Transfer into the container
        else
        {
            if (!this.func_75135_a(itemstack1, 0, containerSlots, false))
            {
                return ItemStack.field_190927_a;
            }
        }

        // Required
        if (itemstack1.func_190916_E() == 0)
        {
            slot.func_75215_d(ItemStack.field_190927_a);
        }
        else
        {
            slot.func_75218_e();
        }
        if (itemstack1.func_190916_E() == itemstack.func_190916_E())
        {
            return ItemStack.field_190927_a;
        }
        slot.func_190901_a(player, itemstack1);
        return itemstack;
    }

    @Override
    @Nonnull
    public ItemStack func_184996_a(int slotID, int dragType, ClickType clickType, EntityPlayer player)
    {
        if ((clickType == ClickType.QUICK_MOVE || clickType == ClickType.PICKUP || clickType == ClickType.SWAP || clickType == ClickType.THROW) && slotID == itemIndex)
        {
            return ItemStack.field_190927_a;
        }
        else
        {
            return super.func_184996_a(slotID, dragType, clickType, player);
        }
    }

    @Override
    public void func_75134_a(EntityPlayer player)
    {
        IItemHandler cap = stack.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null);
        if (cap instanceof ItemStackHandler)
        {
            stack.func_77982_d(((ItemStackHandler) cap).serializeNBT());
        }
        super.func_75134_a(player);
    }

    @Override
    public boolean func_75145_c(EntityPlayer playerIn)
    {
        return true;
    }

    protected abstract void addContainerSlots();

    protected void addPlayerInventorySlots(InventoryPlayer playerInv)
    {
        // Add Player Inventory Slots
        for (int i = 0; i < 3; i++)
        {
            for (int j = 0; j < 9; j++)
            {
                func_75146_a(new Slot(playerInv, j + i * 9 + 9, 8 + j * 18, 84 + i * 18));
            }
        }

        for (int k = 0; k < 9; k++)
        {
            func_75146_a(new Slot(playerInv, k, 8 + k * 18, 142));
        }
    }

}
