/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.entity.ai;

import java.util.HashMap;
import java.util.Map;

import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.entity.passive.EntityAnimal;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import net.dries007.tfc.objects.entity.animal.EntityAnimalOviparous;
import net.dries007.tfc.objects.te.TENestBox;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.calendar.CalendarTFC;
import net.dries007.tfc.util.calendar.ICalendar;

public class EntityAIFindNest extends EntityAIBase
{
    private final double speed;
    private EntityAnimal theCreature;
    private int currentTick;
    private World theWorld;
    private int maxSittingTicks;

    //This is a helper map to prevent chickens not choose unreachable nest boxes.
    private Map<BlockPos, Long> failureDepressionMap;
    private boolean end;

    private BlockPos nestPos = null;

    public EntityAIFindNest(EntityAnimal eAnimal, double speed)
    {
        this.theCreature = eAnimal;
        this.speed = speed;
        this.theWorld = eAnimal.field_70170_p;
        this.failureDepressionMap = new HashMap<>();
        this.func_75248_a(5);
    }

    @Override
    public boolean func_75250_a()
    {
        if (theCreature instanceof EntityAnimalOviparous)
        {
            EntityAnimalOviparous ent = (EntityAnimalOviparous) theCreature;
            return ent.isReadyToLayEggs() && this.getNearbyNest();
        }
        return false;
    }

    @Override
    public boolean func_75253_b()
    {
        if (this.end || !this.isNestBlock(this.theWorld, nestPos))
        {
            this.end = false;
            this.theCreature.func_70661_as().func_75499_g();
            if (this.theCreature.func_184218_aH()) this.theCreature.func_184210_p();
            return false;
        }
        return true;
    }

    @Override
    public void func_75249_e()
    {
        this.theCreature.func_70661_as().func_75492_a(this.nestPos.func_177958_n() + 0.5D, this.nestPos.func_177956_o() + 1, this.nestPos.func_177952_p() + 0.5D, this.speed);
        this.currentTick = 0;
        this.end = false;
        this.maxSittingTicks = this.theCreature.func_70681_au().nextInt(200) + 100;
    }

    @Override
    public void func_75246_d()
    {
        ++this.currentTick;
        if (nestPos == null) return;
        if (this.theCreature.func_174818_b(nestPos) > 1.25D)
        {
            this.theCreature.func_70661_as().func_75492_a(this.nestPos.func_177958_n() + 0.5D, this.nestPos.func_177956_o(), this.nestPos.func_177952_p() + 0.5D, this.speed);
            if (this.currentTick > 200)
            {
                //We never reached it in 10 secs, lets give up on this nest box
                failureDepressionMap.put(nestPos, CalendarTFC.TOTAL_TIME.getTicks() + ICalendar.TICKS_IN_HOUR * 4);
                this.end = true;
            }
        }
        else
        {
            TENestBox te = Helpers.getTE(this.theWorld, nestPos, TENestBox.class);
            if (te != null && theCreature instanceof EntityAnimalOviparous)
            {
                EntityAnimalOviparous ent = (EntityAnimalOviparous) theCreature;
                if (!te.hasBird())
                {
                    te.seatOnThis(ent);
                    this.currentTick = 0;
                }
                if (this.currentTick >= this.maxSittingTicks)
                {
                    NonNullList<ItemStack> eggs = ent.layEggs();
                    for (ItemStack egg : eggs)
                    {
                        te.insertEgg(egg);
                    }
                    this.end = true;
                }
                else if (te.getBird() != ent)
                {
                    //Used by another bird, give up on this one for now
                    failureDepressionMap.put(nestPos, CalendarTFC.TOTAL_TIME.getTicks() + ICalendar.TICKS_IN_HOUR * 4);
                    this.end = true;
                }
            }
        }
    }

    private boolean getNearbyNest()
    {
        int i = (int) this.theCreature.field_70163_u;
        double d0 = Double.MAX_VALUE;
        for (BlockPos.MutableBlockPos pos : BlockPos.func_177975_b(theCreature.func_180425_c().func_177982_a(-16, 0, -16), theCreature.func_180425_c().func_177982_a(16, 4, 16)))
        {
            if (this.isNestBlock(this.theWorld, pos) && this.theWorld.func_175623_d(pos.func_177984_a()))
            {
                double d1 = this.theCreature.func_174818_b(pos);

                if (d1 < d0)
                {
                    this.nestPos = pos.func_185334_h();
                    d0 = d1;
                }
            }
        }
        return d0 < Double.MAX_VALUE;
    }

    private boolean isNestBlock(World world, BlockPos pos)
    {
        if (world == null || pos == null) return false;
        if (failureDepressionMap.containsKey(pos))
        {
            long time = failureDepressionMap.get(pos);
            if (time > CalendarTFC.TOTAL_TIME.getTicks())
                return false;
            else
                failureDepressionMap.remove(pos);
        }
        TENestBox te = Helpers.getTE(world, pos, TENestBox.class);
        return te != null && te.hasFreeSlot() && (te.getBird() == this.theCreature || !te.hasBird());
    }
}
