/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.entity.animal;

import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.world.World;

import net.dries007.tfc.util.calendar.CalendarTFC;

import net.dries007.tfc.objects.entity.animal.EntityAnimalTFC.Gender;

@ParametersAreNonnullByDefault
public abstract class EntityAnimalMammal extends EntityAnimalTFC
{
    private long pregnantTime; //The time(in days) this entity became pregnant

    @SuppressWarnings("unused")
    public EntityAnimalMammal(World worldIn)
    {
        super(worldIn);
    }

    public EntityAnimalMammal(World worldIn, Gender gender, int birthDay)
    {
        super(worldIn, gender, birthDay);
        this.pregnantTime = -1;
    }

    @Override
    public void func_70636_d()
    {
        super.func_70636_d();
        if (!this.field_70170_p.field_72995_K)
        {
            if (this.isFertilized() && CalendarTFC.PLAYER_TIME.getTotalDays() >= pregnantTime + gestationDays())
            {
                birthChildren();
                this.setFertilized(false);
            }
        }
    }

    @Override
    public void func_70014_b(NBTTagCompound nbt)
    {
        super.func_70014_b(nbt);
        nbt.func_74772_a("pregnant", pregnantTime);
    }

    @Override
    public void func_70037_a(NBTTagCompound nbt)
    {
        super.func_70037_a(nbt);
        this.pregnantTime = nbt.func_74763_f("pregnant");
    }

    public void onFertilized(EntityAnimalTFC male)
    {
        //Mark the day this female became pregnant
        this.pregnantTime = CalendarTFC.PLAYER_TIME.getTotalDays();
    }

    /**
     * Spawns children of this animal
     */
    public abstract void birthChildren();

    /**
     * Return the number of days for a full gestation
     *
     * @return long value in days
     */
    public abstract long gestationDays();
}
