/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items.metal;

import javax.annotation.ParametersAreNonnullByDefault;

import com.google.common.collect.Multimap;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.ai.attributes.AttributeModifier;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.EntityEquipmentSlot;
import net.minecraft.item.ItemStack;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.api.capability.damage.DamageType;
import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.util.OreDictionaryHelper;

import net.minecraft.item.Item.ToolMaterial;

@ParametersAreNonnullByDefault
@MethodsReturnNonnullByDefault
public class ItemMetalTool extends ItemMetal
{
    public final ToolMaterial material;
    private final double attackDamage;
    private final int areaOfEffect; // todo: implement
    private final float attackSpeed;
    private float efficiency;

    public ItemMetalTool(Metal metal, Metal.ItemType type)
    {
        super(metal, type);
        if (metal.getToolMetal() == null)
            throw new IllegalArgumentException("You can't make tools out of non tool metals.");
        material = metal.getToolMetal();
        int harvestLevel = material.func_77996_d();

        func_77625_d(1);
        func_77656_e(material.func_77997_a());
        efficiency = material.func_77998_b();

        float typeDamage;
        switch (type)
        {
            case PICK:
                setHarvestLevel("pickaxe", harvestLevel);
                typeDamage = 1.2f; // todo: use some central spot for this (config maybe?) and make the rock equivalents use the same numbers.
                areaOfEffect = 1;
                attackSpeed = -2.8f;
                break;
            case SHOVEL:
                setHarvestLevel("shovel", harvestLevel);
                typeDamage = 1.3f;
                areaOfEffect = 1;
                attackSpeed = -3f;
                break;
            case AXE:
                setHarvestLevel("axe", harvestLevel);
                typeDamage = 1.5f;
                areaOfEffect = 1;
                attackSpeed = -3f;
                OreDictionaryHelper.registerDamageType(this, DamageType.SLASHING);
                break;
            case HOE:
                setHarvestLevel("hoe", harvestLevel);
                typeDamage = 0.7f;
                areaOfEffect = 1;
                attackSpeed = -3;
                break;
            case CHISEL:
                setHarvestLevel("chisel", harvestLevel);
                typeDamage = 0.7f;
                areaOfEffect = 1;
                attackSpeed = 0;
                break;
            case SAW:
                setHarvestLevel("saw", harvestLevel);
                typeDamage = 0.5f;
                areaOfEffect = 1;
                attackSpeed = -1;
                break;
            case PROPICK:
                setHarvestLevel("pickaxe", harvestLevel);
                typeDamage = 1f;
                areaOfEffect = 1;
                attackSpeed = -3.5f;
                func_77656_e(material.func_77997_a() / 3);
                efficiency = material.func_77998_b() * 0.5F;
                break;
            case SCYTHE:
                setHarvestLevel("scythe", harvestLevel);
                func_77656_e((int) (material.func_77997_a() * 1.5));
                typeDamage = 1.5f;
                areaOfEffect = 2;
                attackSpeed = -3.5f;
                break;
            case SHEARS:
                setHarvestLevel("shears", harvestLevel);
                func_77656_e((int) (material.func_77997_a() * 0.3));
                typeDamage = 0.5f;
                areaOfEffect = 1;
                attackSpeed = -3;
                break;
            case KNIFE:
                setHarvestLevel("knife", harvestLevel);
                typeDamage = 0.5f;
                areaOfEffect = 1;
                attackSpeed = 3f;
                OreDictionaryHelper.registerDamageType(this, DamageType.PIERCING);
                break;
            case HAMMER:
                setHarvestLevel("hammer", harvestLevel);
                typeDamage = 2f;
                areaOfEffect = 1;
                attackSpeed = -3.5f;
                OreDictionaryHelper.registerDamageType(this, DamageType.CRUSHING);
                break;
            case SWORD:
                typeDamage = 1f;
                areaOfEffect = 1;
                attackSpeed = -0.75f;
                OreDictionaryHelper.registerDamageType(this, DamageType.SLASHING);
                break;
            case MACE:
                typeDamage = 1.1f;
                areaOfEffect = 1;
                attackSpeed = -1;
                OreDictionaryHelper.registerDamageType(this, DamageType.CRUSHING);
                break;
            case JAVELIN:
                typeDamage = 1f;
                areaOfEffect = 1;
                attackSpeed = -1;
                OreDictionaryHelper.registerDamageType(this, DamageType.PIERCING);
                break;
            case SHIELD:
                typeDamage = 0.1f;
                areaOfEffect = 1;
                attackSpeed = -3;
                break;
            default:
                throw new IllegalArgumentException("Tool from non tool type.");
        }

        attackDamage = typeDamage * material.func_78000_c();
    }

    @Override
    public float func_150893_a(ItemStack stack, IBlockState state)
    {
        return canHarvestBlock(state, stack) ? efficiency : 1.0f;
    }

    @Override
    public boolean func_77644_a(ItemStack stack, EntityLivingBase target, EntityLivingBase attacker)
    {
        switch (type)
        {
            case PROPICK:
            case SAW:
            case SHEARS:
                stack.func_77972_a(4, attacker);
                break;
            case HOE:
            case CHISEL:
                stack.func_77972_a(3, attacker);
                break;
            case PICK:
            case SHOVEL:
            case AXE:
            case SCYTHE:
                stack.func_77972_a(2, attacker);
                break;
            case SWORD:
            case MACE:
            case JAVELIN:
            case HAMMER:
            case KNIFE:
                stack.func_77972_a(1, attacker);
                break;
        }
        return true;
    }

    @Override
    public boolean func_179218_a(ItemStack stack, World worldIn, IBlockState state, BlockPos pos, EntityLivingBase entityLiving)
    {
        if (state.func_185887_b(worldIn, pos) > 0 || type == Metal.ItemType.KNIFE || type == Metal.ItemType.SCYTHE)
        {
            if (!worldIn.field_72995_K)
            {
                stack.func_77972_a(1, entityLiving);
            }
        }
        if (areaOfEffect > 1 && entityLiving instanceof EntityPlayer && !worldIn.field_72995_K)
        {
            EntityPlayer player = (EntityPlayer) entityLiving;
            int areaPlus = areaOfEffect - 1; //First block already added
            for (BlockPos.MutableBlockPos extraPos : BlockPos.func_177975_b(pos.func_177982_a(-areaPlus, -areaPlus, -areaPlus), pos.func_177982_a(areaPlus, areaPlus, areaPlus)))
            {
                IBlockState st = worldIn.func_180495_p(extraPos);
                if (!extraPos.equals(pos) && !worldIn.func_175623_d(extraPos) && func_150897_b(st))
                {
                    st.func_177230_c().func_176206_d(worldIn, extraPos, st);
                    st.func_177230_c().func_180657_a(worldIn, player, extraPos, st, worldIn.func_175625_s(extraPos), stack);
                    worldIn.func_175698_g(extraPos);
                    stack.func_77972_a(1, entityLiving);
                }
            }
        }
        return true;
    }

    @Override
    public boolean func_150897_b(IBlockState state)
    {
        Material material = state.func_185904_a();
        switch (type)
        {
            case AXE:
                return material == Material.field_151575_d || material == Material.field_151585_k || material == Material.field_151582_l;
            case PICK:
                return material == Material.field_151573_f || material == Material.field_151574_g || material == Material.field_151576_e;
            case SHOVEL:
                return material == Material.field_151597_y || material == Material.field_151596_z;
            case SCYTHE:
                return material == Material.field_151585_k || material == Material.field_151582_l || material == Material.field_151584_j;
        }
        return false;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public boolean func_77662_d()
    {
        return true;
    }

    @Override
    public int func_77619_b()
    {
        return material.func_77995_e();
    }

    @Override
    public Multimap<String, AttributeModifier> getAttributeModifiers(EntityEquipmentSlot slot, ItemStack stack)
    {
        Multimap<String, AttributeModifier> multimap = super.getAttributeModifiers(slot, stack);
        if (slot == EntityEquipmentSlot.MAINHAND)
        {
            multimap.put(SharedMonsterAttributes.field_111264_e.func_111108_a(), new AttributeModifier(field_111210_e, "Tool modifier", attackDamage, 0));
            multimap.put(SharedMonsterAttributes.field_188790_f.func_111108_a(), new AttributeModifier(field_185050_h, "Tool modifier", attackSpeed, 0));
        }
        return multimap;
    }

    @Override
    public boolean canDestroyBlockInCreative(World world, BlockPos pos, ItemStack stack, EntityPlayer player)
    {
        //This stops swords and other weapons breaking blocks in creative
        return func_150897_b(world.func_180495_p(pos));
    }

    @Override
    public boolean canHarvestBlock(IBlockState state, ItemStack stack)
    {
        for (String type : getToolClasses(stack))
        {
            if (state.func_177230_c().isToolEffective(type, state))
            {
                return true;
            }
        }
        return func_150897_b(state);
    }

    @Override
    public boolean canStack(ItemStack stack)
    {
        return false;
    }

    @Override
    public boolean doesSneakBypassUse(ItemStack stack, IBlockAccess world, BlockPos pos, EntityPlayer player)
    {
        // Hammers need to activate anvils for welding
        return this.type == Metal.ItemType.HAMMER || super.doesSneakBypassUse(stack, world, pos, player);
    }

    public double getAttackDamage() { return this.attackDamage; }
}
