/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.items.metal;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.*;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.TextComponentString;
import net.minecraft.util.text.TextComponentTranslation;
import net.minecraft.world.World;

import net.dries007.tfc.ConfigTFC;
import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.world.classic.worldgen.vein.VeinRegistry;
import net.dries007.tfc.world.classic.worldgen.vein.VeinType;

public class ItemProspectorPick extends ItemMetalTool
{
    private static final int PROSPECT_RADIUS = 12;
    // todo: balance. 40 ticks feels really long, especially for the preciseness you want from the propick
    private static final int COOLDOWN = 10;
    private static final Random RANDOM = new Random();

    public ItemProspectorPick(Metal metal, Metal.ItemType type)
    {
        super(metal, type);
    }

    @Override
    @Nonnull
    public EnumActionResult func_180614_a(EntityPlayer player, World worldIn, BlockPos pos, EnumHand hand, @Nullable EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        IBlockState state = worldIn.func_180495_p(pos);
        if (facing != null)
        {
            SoundType soundType = state.func_177230_c().getSoundType(state, worldIn, pos, player);
            worldIn.func_184133_a(player, pos, soundType.func_185846_f(), SoundCategory.BLOCKS, 1.0f, soundType.func_185847_b());

            if (!worldIn.field_72995_K)
            {
                // Damage item and set cooldown
                player.func_184586_b(hand).func_77972_a(1, player);
                player.func_184811_cZ().func_185145_a(this, COOLDOWN);

                RANDOM.setSeed(pos.func_177986_g());
                ItemStack targetStack = getOreStack(state, false);
                if (targetStack != null)
                {
                    // Just clicked on an ore block
                    player.func_146105_b(new TextComponentTranslation("tfc.propick.found").func_150258_a(" " + targetStack.func_82833_r()), ConfigTFC.CLIENT.propickOutputToActionBar);
                }
                else if (RANDOM.nextFloat() < 0.4)
                {
                    // False negative
                    player.func_146105_b(new TextComponentTranslation("tfc.propick.found_nothing"), ConfigTFC.CLIENT.propickOutputToActionBar);
                }
                else
                {
                    Collection<ProspectResult> results = scanSurroundingBlocks(worldIn, pos);
                    if (results.isEmpty())
                    {
                        // Found nothing
                        player.func_146105_b(new TextComponentTranslation("tfc.propick.found_nothing"), ConfigTFC.CLIENT.propickOutputToActionBar);
                    }
                    else
                    {
                        // Found something
                        ProspectResult result = (ProspectResult) results.toArray()[RANDOM.nextInt(results.size())];

                        String translationKey;
                        if (result.score < 10)
                        {
                            translationKey = "tfc.propick.found_traces";
                        }
                        else if (result.score < 20)
                        {
                            translationKey = "tfc.propick.found_small";
                        }
                        else if (result.score < 40)
                        {
                            translationKey = "tfc.propick.found_medium";
                        }
                        else if (result.score < 80)
                        {
                            translationKey = "tfc.propick.found_large";
                        }
                        else
                        {
                            translationKey = "tfc.propick.found_very_large";
                        }

                        player.func_146105_b(new TextComponentTranslation(translationKey).func_150258_a(" ").func_150257_a(new TextComponentTranslation(result.ore.func_82833_r())), ConfigTFC.CLIENT.propickOutputToActionBar);

                        if (ConfigTFC.GENERAL.debug)
                        {
                            for (int i = 0; i < results.size(); i++)
                            {
                                player.func_146105_b(new TextComponentString(result.ore.func_82833_r() + ": " + String.format("%.02f", result.score)), false);
                            }
                        }
                    }
                }
            }
        }
        else
        {
            //client side, add hit particles
            addHitBlockParticle(worldIn, pos, facing, state);
        }

        return EnumActionResult.SUCCESS;
    }

    /**
     * Loops through every block in a 25x25x25 cube around the center
     *
     * @param world  The world
     * @param center The center position
     * @return the collection of results
     */
    @Nonnull
    private Collection<ProspectResult> scanSurroundingBlocks(World world, BlockPos center)
    {
        Map<String, ProspectResult> results = new HashMap<>();
        for (BlockPos.MutableBlockPos pos : BlockPos.MutableBlockPos.func_177975_b(center.func_177982_a(-PROSPECT_RADIUS, -PROSPECT_RADIUS, -PROSPECT_RADIUS), center.func_177982_a(PROSPECT_RADIUS, PROSPECT_RADIUS, PROSPECT_RADIUS)))
        {
            ItemStack stack = getOreStack(world.func_180495_p(pos), true);
            if (stack != null)
            {
                String oreName = stack.func_82833_r();
                if (results.containsKey(oreName))
                {
                    results.get(oreName).score += 1;
                }
                else
                {
                    results.put(oreName, new ProspectResult(stack, 1));
                }
            }
        }
        return results.values();
    }

    @Nullable
    private ItemStack getOreStack(IBlockState blockState, boolean ignoreGrade)
    {
        if (blockState == null || BlocksTFC.isGround(blockState))
        {
            return null;
        }
        for (VeinType vein : VeinRegistry.INSTANCE.getVeins().values())
            if (vein.isOreBlock(blockState))
            {
                Block block = blockState.func_177230_c();
                if (vein.ore != null)
                    if (vein.ore.isGraded() && !ignoreGrade)
                        return new ItemStack(block.func_180660_a(blockState, null, 0), 1, block.func_176201_c(blockState));
                    else
                        return new ItemStack(block.func_180660_a(blockState, null, 0), 1, 0);
                else
                    return new ItemStack(Item.func_150898_a(block), 1, block.func_176201_c(blockState));
            }
        return null;
    }

    private void addHitBlockParticle(World world, BlockPos pos, EnumFacing side, IBlockState state)
    {
        int x = pos.func_177958_n();
        int y = pos.func_177956_o();
        int z = pos.func_177952_p();
        AxisAlignedBB axisalignedbb = state.func_185900_c(world, pos);
        for (int i = 0; i < 2; i++)
        {
            double xOffset = x + RANDOM.nextDouble() * (axisalignedbb.field_72336_d - axisalignedbb.field_72340_a - 0.2D) + 0.1D + axisalignedbb.field_72340_a;
            double yOffset = y + RANDOM.nextDouble() * (axisalignedbb.field_72337_e - axisalignedbb.field_72338_b - 0.2D) + 0.1D + axisalignedbb.field_72338_b;
            double zOffset = z + RANDOM.nextDouble() * (axisalignedbb.field_72334_f - axisalignedbb.field_72339_c - 0.2D) + 0.1D + axisalignedbb.field_72339_c;

            switch (side)
            {
                case WEST:
                    xOffset = x + axisalignedbb.field_72340_a - 0.1D;
                    break;
                case EAST:
                    xOffset = x + axisalignedbb.field_72336_d + 0.1D;
                    break;
                case DOWN:
                    yOffset = y + axisalignedbb.field_72338_b - 0.1D;
                    break;
                case UP:
                    yOffset = y + axisalignedbb.field_72337_e + 0.1D;
                    break;
                case NORTH:
                    zOffset = z + axisalignedbb.field_72339_c - 0.1D;
                    break;
                case SOUTH:
                    zOffset = z + axisalignedbb.field_72334_f + 0.1D;
                    break;
            }

            world.func_175688_a(EnumParticleTypes.BLOCK_CRACK, xOffset, yOffset, zOffset, 0.0D, 0.0D, 0.0D, Block.func_176210_f(state));
        }
    }

    private static final class ProspectResult
    {
        private final ItemStack ore;
        private double score;

        ProspectResult(ItemStack itemStack, double num)
        {
            ore = itemStack;
            score = num;
        }

    }
}
