/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.recipes;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import com.google.gson.JsonObject;
import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.world.World;
import net.minecraftforge.common.crafting.IRecipeFactory;
import net.minecraftforge.common.crafting.JsonContext;
import net.minecraftforge.registries.IForgeRegistryEntry;

import net.dries007.tfc.api.capability.food.CapabilityFood;
import net.dries007.tfc.api.capability.food.IFood;

@ParametersAreNonnullByDefault
public class FoodStackingRecipe extends IForgeRegistryEntry.Impl<IRecipe> implements IRecipe
{
    @Override
    public boolean func_77569_a(InventoryCrafting inv, World worldIn)
    {
        ItemStack foodStack = ItemStack.field_190927_a;
        for (int i = 0; i < inv.func_70302_i_(); i++)
        {
            ItemStack stack = inv.func_70301_a(i);
            if (!stack.func_190926_b())
            {
                // All nonempty stacks must be a food
                if (!stack.hasCapability(CapabilityFood.CAPABILITY, null))
                {
                    return false;
                }
                // If there's more than one stack, keep count of the previous ones
                if (foodStack.func_190926_b())
                {
                    // First stack, so save and move on
                    foodStack = stack;
                }
                else
                {
                    // Another stack, so compare. If not equal, or one is not a food, the recipe is invalid
                    if (!foodStack.func_77969_a(stack))
                    {
                        return false;
                    }
                }
            }
        }
        return true;
    }

    @Override
    @Nonnull
    public ItemStack func_77572_b(InventoryCrafting inv)
    {
        ItemStack resultStack = ItemStack.field_190927_a;
        int outputAmount = 0;
        long minCreationDate = -1;
        for (int i = 0; i < inv.func_70302_i_(); i++)
        {
            ItemStack stack = inv.func_70301_a(i);
            if (!stack.func_190926_b())
            {
                // Get the food capability
                IFood cap = stack.getCapability(CapabilityFood.CAPABILITY, null);
                if (cap != null)
                {
                    // Increment output amount
                    outputAmount++;

                    // Find the min creation date
                    if (minCreationDate == -1 || minCreationDate > cap.getCreationDate())
                    {
                        minCreationDate = cap.getCreationDate();
                    }

                    // And save the stack
                    if (resultStack.func_190926_b())
                    {
                        resultStack = stack.func_77946_l();
                    }
                }
            }
        }

        // Update the capability and count of the result
        resultStack.func_190920_e(outputAmount);
        IFood cap = resultStack.getCapability(CapabilityFood.CAPABILITY, null);
        if (cap != null)
        {
            cap.setCreationDate(minCreationDate);
        }
        return resultStack;
    }

    @Override
    public boolean func_194133_a(int width, int height)
    {
        return true;
    }

    @Override
    @Nonnull
    public ItemStack func_77571_b()
    {
        return ItemStack.field_190927_a;
    }

    @Override
    public boolean func_192399_d()
    {
        return true;
    }

    @SuppressWarnings("unused")
    public static class Factory implements IRecipeFactory
    {
        @Override
        public IRecipe parse(JsonContext context, JsonObject json)
        {
            return new FoodStackingRecipe();
        }
    }
}
