/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.state.IBlockState;
import net.minecraft.inventory.InventoryHelper;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;

import net.dries007.tfc.api.types.Metal;
import net.dries007.tfc.objects.blocks.metal.BlockMetalSheet;
import net.dries007.tfc.objects.items.metal.ItemMetalSheet;

@ParametersAreNonnullByDefault
public class TEMetalSheet extends TEBase
{
    private boolean[] faces;

    public TEMetalSheet()
    {
        this.faces = new boolean[6];
    }

    /**
     * Gets the number of faces that are present
     *
     * @return a number in [0, 6]
     */
    public int getFaceCount()
    {
        int n = 0;
        for (boolean b : faces)
        {
            if (b) n++;
        }
        return n;
    }

    /**
     * Checks if sheet is present for the given face
     *
     * @param face The face to check
     * @return true if present
     */
    public boolean getFace(EnumFacing face)
    {
        return faces[face.func_176745_a()];
    }

    public void setFace(EnumFacing facing, boolean value)
    {
        if (!field_145850_b.field_72995_K)
        {
            faces[facing.func_176745_a()] = value;
            func_70296_d();
            IBlockState state = field_145850_b.func_180495_p(field_174879_c);
            for (EnumFacing face : EnumFacing.values())
            {
                state = state.func_177226_a(BlockMetalSheet.FACE_PROPERTIES[face.func_176745_a()], faces[face.func_176745_a()]);
            }
            field_145850_b.func_175656_a(field_174879_c, state);
        }
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt)
    {
        for (EnumFacing face : EnumFacing.values())
        {
            faces[face.func_176745_a()] = nbt.func_74767_n(face.func_176610_l());
        }
        super.func_145839_a(nbt);
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189515_b(NBTTagCompound nbt)
    {
        for (EnumFacing face : EnumFacing.values())
        {
            nbt.func_74757_a(face.func_176610_l(), faces[face.func_176745_a()]);
        }
        return super.func_189515_b(nbt);
    }

    public void onBreakBlock(Metal outMetal)
    {
        Item item = ItemMetalSheet.get(outMetal, Metal.ItemType.SHEET);
        ItemStack output = new ItemStack(item, getFaceCount());
        InventoryHelper.func_180173_a(field_145850_b, field_174879_c.func_177958_n(), field_174879_c.func_177956_o(), field_174879_c.func_177952_p(), output);
    }
}
