/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.te;

import javax.annotation.Nonnull;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.passive.EntityCow;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.ITickable;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;

import net.dries007.tfc.api.recipes.QuernRecipe;
import net.dries007.tfc.objects.items.ItemHandstone;
import net.dries007.tfc.objects.items.ItemsTFC;

import static net.minecraft.init.SoundEvents.*;

@ParametersAreNonnullByDefault
public class TEQuern extends TEInventory implements ITickable
{
    public static final int SLOT_HANDSTONE = 0;
    public static final int SLOT_INPUT = 1;
    public static final int SLOT_OUTPUT = 2;

    private int rotationTimer;
    private boolean hasHandstone;

    public TEQuern()
    {
        super(3);
        rotationTimer = 0;
    }

    public ItemStack insertOrSwapItem(int slot, ItemStack playerStack)
    {
        ItemStack quernStack = inventory.getStackInSlot(slot);

        if (quernStack.func_190926_b() || (playerStack.func_77985_e() && quernStack.func_77985_e() && quernStack.func_77973_b() == playerStack.func_77973_b() && (!playerStack.func_77981_g() || playerStack.func_77960_j() == quernStack.func_77960_j()) && ItemStack.func_77970_a(playerStack, quernStack)))
        {
            return inventory.insertItem(slot, playerStack, false);
        }
        inventory.setStackInSlot(slot, playerStack);
        return quernStack;
    }

    @Override
    public int getSlotLimit(int slot)
    {
        return slot == SLOT_HANDSTONE ? 1 : 64;
    }

    @Override
    public boolean isItemValid(int slot, ItemStack stack)
    {
        switch (slot)
        {
            case SLOT_HANDSTONE:
                return stack.func_77973_b() instanceof ItemHandstone;
            case SLOT_INPUT:
                return QuernRecipe.get(stack) != null;
            default:
                return false;
        }
    }

    @Override
    public void setAndUpdateSlots(int slot)
    {
        updateBlock();
        if (slot == SLOT_HANDSTONE)
        {
            hasHandstone = inventory.getStackInSlot(SLOT_HANDSTONE).func_77973_b() instanceof ItemHandstone;
        }
        super.setAndUpdateSlots(slot);
    }

    @Override
    public void func_145839_a(NBTTagCompound nbt)
    {
        rotationTimer = nbt.func_74762_e("rotationTimer");
        super.func_145839_a(nbt);
        hasHandstone = inventory.getStackInSlot(SLOT_HANDSTONE).func_77973_b() instanceof ItemHandstone;
    }

    @Override
    @Nonnull
    public NBTTagCompound func_189515_b(NBTTagCompound nbt)
    {
        nbt.func_74768_a("rotationTimer", rotationTimer);
        return super.func_189515_b(nbt);
    }

    @Override
    public boolean canInteractWith(EntityPlayer player)
    {
        return super.canInteractWith(player) && rotationTimer == 0;
    }

    public int getRotationTimer()
    {
        return rotationTimer;
    }

    public boolean isGrinding()
    {
        return rotationTimer > 0;
    }

    public boolean hasHandstone()
    {
        return hasHandstone;
    }

    public void grind()
    {
        this.rotationTimer = 90;
        updateBlock();
    }

    @Override
    public void func_73660_a()
    {
        if (rotationTimer > 0)
        {
            rotationTimer--;

            if (rotationTimer == 0)
            {
                grindItem();
                field_145850_b.func_184133_a(null, field_174879_c, field_187704_k, SoundCategory.BLOCKS, 1.0f, 0.8f);
                inventory.getStackInSlot(SLOT_HANDSTONE).func_77972_a(1, new EntityCow(field_145850_b));

                if (inventory.getStackInSlot(SLOT_HANDSTONE).func_190926_b())
                {
                    for (int i = 0; i < 15; i++)
                    {
                        field_145850_b.func_175688_a(EnumParticleTypes.ITEM_CRACK, field_174879_c.func_177958_n() + 0.5D, field_174879_c.func_177956_o() + 0.875D, field_174879_c.func_177952_p() + 0.5D, (field_145850_b.field_73012_v.nextDouble() - field_145850_b.field_73012_v.nextDouble()) / 4, field_145850_b.field_73012_v.nextDouble() / 4, (field_145850_b.field_73012_v.nextDouble() - field_145850_b.field_73012_v.nextDouble()) / 4, Item.func_150891_b(ItemsTFC.HANDSTONE));
                    }
                    field_145850_b.func_184133_a(null, field_174879_c, field_187835_fT, SoundCategory.BLOCKS, 1.0f, 0.8f);
                    field_145850_b.func_184133_a(null, field_174879_c, field_187635_cQ, SoundCategory.BLOCKS, 1.0f, 0.6f);
                }

                setAndUpdateSlots(SLOT_HANDSTONE);
            }
        }
    }

    @Nonnull
    @Override
    public AxisAlignedBB getRenderBoundingBox()
    {
        return new AxisAlignedBB(func_174877_v(), func_174877_v().func_177982_a(1, 2, 1));
    }

    public ItemStack takeCraftingResult(ItemStack stack)
    {
        return inventory.extractItem(SLOT_OUTPUT, stack.func_190916_E(), false);
    }

    public void updateBlock()
    {
        IBlockState state = field_145850_b.func_180495_p(field_174879_c);
        field_145850_b.func_184138_a(field_174879_c, state, state, 3);
        func_70296_d();
    }

    private void grindItem()
    {
        ItemStack inputStack = inventory.getStackInSlot(SLOT_INPUT);
        if (!inputStack.func_190926_b())
        {
            ItemStack resultStack = QuernRecipe.get(inputStack).getOutputItem(inputStack);
            ItemStack outputStack = inventory.getStackInSlot(SLOT_OUTPUT);

            if (outputStack.func_190926_b())
            {
                inventory.setStackInSlot(SLOT_OUTPUT, resultStack.func_77946_l());

                inputStack.func_190918_g(1);
            }
            else if (outputStack.func_77973_b() == resultStack.func_77973_b())
            {
                if (outputStack.func_190916_E() + resultStack.func_190916_E() <= outputStack.func_77976_d())
                {
                    inventory.insertItem(SLOT_OUTPUT, resultStack, false);

                    inputStack.func_190918_g(1);
                }
            }
        }
    }
}
